@@ -41,7 +41,7 @@ try:
     purepress_config = toml.load(os.path.join(root_folder, "purepress.toml"))
 except FileNotFoundError:
     purepress_config = {"site": {}, "config": {}}
-site, user_config = purepress_config["site"], purepress_config["config"]
+site, config = purepress_config["site"], purepress_config["config"]
 
 app = Flask(
     __name__,
@@ -63,25 +63,55 @@ app.register_blueprint(theme_bp)
 # prepare markdown parser
 class HookImageSrcProcessor(markdown.treeprocessors.Treeprocessor):
     def run(self, root: etree.Element):
-        # TODO: hook all relative links
+        static_url = url_for("static", filename="")
         for el in root.iter("img"):
             src = el.get("src", "")
-            static_url = url_for("static", filename="", _external=True)
-            el.set("src", re.sub(r"^/static/", static_url, src))
+            if src.startswith("/static/"):
+                el.set("src", re.sub(r"^/static/", static_url, src))
 
 
-class HookImageSrcExtension(markdown.extensions.Extension):
+class HookLinkHrefProcessor(markdown.treeprocessors.Treeprocessor):
+    @staticmethod
+    def path_to_url(path: str) -> str:
+        root = url_for("index").rstrip("/")
+        url = path
+        if path.startswith("/posts/"):
+            # /posts/2021-08-23-hello-world.md -> /post/2021/08/23/hello-world/
+            url = re.sub(r"^/posts/", f"{root}/post/", url)
+            url = re.sub(r"-", "/", url, count=3)
+            url = re.sub(r"\.md$", "/", url)
+        elif path.startswith("/pages/"):
+            # /pages/about/ -> /about/
+            # /pages/about/index.md -> /about/
+            # /pages/foo/bar.md -> /foo/bar.html
+            url = re.sub(r"^/pages/", f"{root}/", url)
+            url = re.sub(r"index\.md$", "", url)
+            url = re.sub(r"\.md$", ".html", url)
+        elif path.startswith("/raw/"):
+            # /raw/foo/baz.html -> /foo/baz.html
+            url = re.sub(r"^/raw/", f"{root}/", url)
+        return url
+
+    def run(self, root: etree.Element):
+        for el in root.iter("a"):
+            href = el.get("href", "")
+            if href.startswith("/"):
+                el.set("href", self.path_to_url(href))
+
+
+class Extension(markdown.extensions.Extension):
     def extendMarkdown(self, md) -> None:
         md.treeprocessors.register(HookImageSrcProcessor(), "hook-image-src", 5)
+        md.treeprocessors.register(HookLinkHrefProcessor(), "hook-link-href", 5)
 
 
-md = Markdown(extensions=[GithubFlavoredMarkdownExtension(), HookImageSrcExtension()])
+md = Markdown(extensions=[GithubFlavoredMarkdownExtension(), Extension()])
 
 
-# inject site and user_config into template context
+# inject site and config into template context
 @app.context_processor
 def inject_objects() -> Dict[str, Any]:
-    return {"site": site, "user_config": user_config}
+    return {"global": {"site": site, "config": config}}
 
 
 def load_entry(
@@ -117,7 +147,7 @@ def load_entry(
             parser = HtmlTocParser()
             parser.feed(entry["content"])
             entry["content"] = parser.html
-            depth = entry.get("toc_depth", user_config.get("toc_depth")) or 0
+            depth = entry.get("toc_depth", config.get("toc_depth")) or 0
             entry["toc"] = parser.toc(depth=depth)
             entry["toc_html"] = parser.toc_html(depth=depth)
     return entry
@@ -225,7 +255,7 @@ def index():
 @templated("index")
 def index_page(page_num, *, from_index: bool = False):
     # do some calculation and handle unexpected cases
-    posts_per_page = user_config["posts_per_index_page"]
+    posts_per_page = config["posts_per_index_page"]
     posts = load_posts(meta_only=True)  # just load meta data quickly
     post_count = len(posts)
     page_count = (post_count + posts_per_page - 1) // posts_per_page
